@@ -9,28 +9,33 @@ typedef struct srl_iterator         * srl_iterator_ptr;
 typedef struct srl_iterator         srl_iterator_t;
 typedef struct srl_iterator_stack   srl_iterator_stack_t;
 typedef struct srl_iterator_stack   * srl_iterator_stack_ptr;
-typedef struct srl_stack            * srl_stack_ptr;
+
+struct srl_iterator_stack {
+    I32 idx;        // index of current object, in negative format
+    U32 count;      // number of child objects
+    UV offset;      // offset of the tag
+    U8 tag;
+};
+
+#define srl_stack_type_t srl_iterator_stack_t
+#include "srl_stack.h"
 
 /* the iterator main struct */
 struct srl_iterator {
     srl_reader_buffer_t buf;
     srl_reader_buffer_ptr pbuf;
-    srl_stack_ptr stack;
-    SV *tmp_buf_owner;
+    srl_stack_t stack;
+    srl_stack_ptr pstack;
     SV *document;
     struct srl_decoder *dec;
 };
 
-struct srl_iterator_stack {
-    UV offset;      // offset of the tag
-    U32 count;      // number of child objects
-    I32 idx;        // index of current object, in negative format
-    U8 tag;
-};
-
 /* constructor/destructor */
-srl_iterator_t *srl_build_iterator_struct(pTHX_ HV *opt);
-void srl_destroy_iterator(pTHX_ srl_iterator_t *iter);
+srl_iterator_t *srl_build_iterator_struct(pTHX_ HV *opt);    // allocate structure and initalize
+void srl_init_iterator(pTHX_ srl_iterator_t *iter, HV *opt); // initialize structure
+void srl_deinit_iterator(pTHX_ srl_iterator_t *iter);        // deinitalize structure without freeing it
+void srl_destroy_iterator(pTHX_ srl_iterator_t *iter);       // destroy structure and free it
+void srl_copy_iterator(pTHX_ srl_iterator_t *from, srl_iterator_t *to);
 
 /* Sereal document */
 void srl_iterator_set(pTHX_ srl_iterator_t *iter, SV *src);
@@ -59,6 +64,7 @@ UV srl_iterator_object_info(pTHX_ srl_iterator_t *iter, UV *length_ptr);
 
 /* array parsing */
 IV srl_iterator_array_goto(pTHX_ srl_iterator_t *iter, I32 idx);
+IV srl_iterator_array_exists(pTHX_ srl_iterator_t *iter, I32 idx);
 
 /* hash parsing */
 const char * srl_iterator_hash_key(pTHX_ srl_iterator_t *iter, STRLEN *len_out);
@@ -68,6 +74,8 @@ IV srl_iterator_hash_exists_sv(pTHX_ srl_iterator_t *iter, SV *name);
 
 SV * srl_iterator_decode(pTHX_ srl_iterator_t *iter); // return mortalized SV
 
+#define SRL_ITER_NOT_FOUND (-1)
+
 #define SRL_ITERATOR_OBJ_IS_SCALAR  (1 << 1)
 #define SRL_ITERATOR_OBJ_IS_ARRAY   (1 << 2)
 #define SRL_ITERATOR_OBJ_IS_HASH    (1 << 3)
